from django.utils import unittest

from test_app.models import Author, Book, FeaturedAuthor


class StatusableModelTestCase(unittest.TestCase):

    def setUp(self):

        # Create some test authors

        self.author1 = Author.objects.create(first_name='Dave', last_name='Eggers')
        self.author2 = Author.objects.create(first_name='George', last_name='Martin')
        self.author3 = Author.objects.create(first_name='Noam', last_name='Chomsky')

        # Create some test books

        self.book1 = Book.objects.create(name='A Heartbreaking Work of Staggering Genius', author=self.author1)
        self.book2 = Book.objects.create(name='You Shall Know Our Velocity', author=self.author1)
        self.book3 = Book.objects.create(name='A Game of Thrones', author=self.author2)
        self.book4 = Book.objects.create(name='Manufacturing Consent: The Political Economy of the Mass Media', author=self.author3)

        # Create some test featured authors

        self.featured1 = FeaturedAuthor.objects.create(author=self.author1)
        self.featured2 = FeaturedAuthor.objects.create(author=self.author2)
        self.featured3 = FeaturedAuthor.objects.create(author=self.author3)

    def tearDown(self):
        
        self.author1.delete()
        self.author2.delete()
        self.author3.delete()

        self.book1.delete()
        self.book2.delete()
        self.book3.delete()
        self.book4.delete()

        self.featured1.delete()
        self.featured2.delete()
        self.featured3.delete()

    def test_inactive(self):

        Author.objects.deactivate()

        # Test queryset methods

        active_authors = Author.objects.active()
        inactive_authors = Author.objects.inactive()

        self.assertEqual(active_authors.count(), 0)
        self.assertEqual(inactive_authors.count(), 3)

        # Test properties

        for author in inactive_authors:
            self.assertFalse(author.is_active)
            self.assertTrue(author.is_inactive)

    def test_active(self):

        Author.objects.activate()

        # Test queryset methods

        active_authors = Author.objects.active()
        inactive_authors = Author.objects.inactive()

        self.assertEqual(active_authors.count(), 3)
        self.assertEqual(inactive_authors.count(), 0)

        # Test properties

        for author in active_authors:
            self.assertTrue(author.is_active)
            self.assertFalse(author.is_inactive)

    def test_instance(self):

        # Test instance methods

        self.author1.activate()
        self.assertTrue(self.author1.is_active)
        self.assertFalse(self.author1.is_inactive)

        self.author1.deactivate(save=False)
        self.assertTrue(self.author1.is_inactive)
        self.assertFalse(self.author1.is_active)

        author1 = Author.objects.get(pk=self.author1.pk)
        self.assertTrue(author1.is_active)
        self.assertFalse(author1.is_inactive)

    def test_single_active(self):

        # Simple activation
        
        self.featured1.activate()
        self.assertTrue(self.featured1.is_active)
        self.assertFalse(self.featured1.is_inactive)
        self.assertIsInstance(FeaturedAuthor.objects.active(), FeaturedAuthor)

        # Activate another record

        self.featured2.activate()

        self.featured1 = FeaturedAuthor.objects.get(pk=self.featured1.pk) # Reload from DB

        self.assertFalse(self.featured1.is_active)
        self.assertTrue(self.featured1.is_inactive)
        self.assertTrue(self.featured2.is_active)
        self.assertFalse(self.featured2.is_inactive)
        self.assertIsInstance(FeaturedAuthor.objects.active(), FeaturedAuthor)

        # Test activating a group

        featured = FeaturedAuthor.objects.all()
        active = featured[0]
        featured = featured.activate()

        i = 0

        featured_updated = FeaturedAuthor.objects.all()

        for feature in featured_updated:
            if (feature.pk == active.pk):
                self.assertTrue(feature.is_active)
            else:
                self.assertFalse(feature.is_active)
            i += 1

        self.assertIsInstance(FeaturedAuthor.objects.active(), FeaturedAuthor)




from django.db import models
from django.forms import forms

class RestrictedTypeFileField(models.FileField):

    """
    Extension of FileField used to restrict to specified file types.
    accepted_types: a list of allowed content types (application/pdf, image/jpeg, image/png, etc)
    """

    def __init__(self, *args, **kwargs):

        self.accepted_types = kwargs.pop("accepted_types", [])
        super(RestrictedTypeFileField, self).__init__(*args, **kwargs)


    def clean(self, *args, **kwargs):

        data = super(RestrictedTypeFileField, self).clean(*args, **kwargs)
        file = data.file
        try:
            content_type = file.content_type
            if not content_type in self.accepted_types:
                raise forms.ValidationError('Filetype not supported.')
        except AttributeError:
            pass

        return data

try:
    # custom fields need to be explicitly introspected by South:
    # http://south.readthedocs.org/en/latest/customfields.html
    from south.modelsinspector import add_introspection_rules

    add_introspection_rules([
        (
            [RestrictedTypeFileField], 
            [],         # Positional args
            {           # Kwargs
                "accepted_types": ["accepted_types", {}],
            },
        ),
    ], ["^geetar\.fields\.RestrictedTypeFileField"])
except Exception:
    pass




from django.db import models

from statusable.models import StatusableModel


class GeetarBaseModel(StatusableModel):

    """
    Handy base model with status and date info, would likely serve well
    as a base model in most cases
    """

    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True



from django.utils import unittest

from geetar import google_maps


class GoogleMapsTestCase(unittest.TestCase):

    def test_find_geo(self):
        
        # Valid locations
        
        locations = [
            '7719 N. McKenna Ave., Portland, OR',
            'Los Angeles, CA',
            '90210'
        ]
        
        for location in locations:
            self.assertIsInstance(google_maps.find_geo(location), dict, "Valid address doesn't return a dict object: %s" % location)
        
        # Invalid locations

        empty = google_maps.find_geo('')
        self.assertEqual(empty, None, "Blank location doesn't return None: %s" % str(empty))

    def test_find_geo_point(self):

        # Valid locations

        locations = [
            '419 NE 10th Ave., Portland, OR 97232',
            'Mexico',
            'New York, NY',
            'Illinois'
        ]

        for location in locations:
            result = google_maps.find_geo_point(location)
            self.assertIsInstance(result, tuple, "Valid address doesn't return a tuple: %s, returns: %s" % (location, str(result)))
            self.assertIsInstance(result[0], (str, unicode,), "First tuple element is not a string for location: %s, returns: %s" % (location, type(result[0])))
            self.assertIsInstance(result[1][0], float, "2nd tuple element doesn't have a float for first item: %s, returns: %s" % (location, type(result[1][0])))
            self.assertIsInstance(result[1][1], float, "First tuple element is not a string for location: %s, returns: %s" % (location, type(result[0])))

        # Invalid locations

        invalid_locations = [
            '',
            'sfhrgurhkj'
        ]

        for location in invalid_locations:
            result = google_maps.find_geo_point(location)
            self.assertEqual(result, None)



from django.utils import unittest
from django.utils.encoding import force_unicode

from base import TemplateTestCase

from geetar.templatetags.geetar import even, odd, split, possessive, jsonify, \
                                       paragraphlist, multibyte_truncate


class SimpleFiltersTestCase(TemplateTestCase):

    global_template_string = '{% load geetar %}'

    def test_odd_even(self):

        # Raw

        self.assertTrue(even(2))
        self.assertTrue(even(10))
        self.assertTrue(even(84))
        self.assertTrue(even(-48))
        self.assertFalse(even(3))
        self.assertFalse(even(69))
        self.assertFalse(even(1001))
        self.assertFalse(even(-175))

        self.assertTrue(odd(1))
        self.assertTrue(odd(11))
        self.assertTrue(odd(1001))
        self.assertTrue(odd(-13))
        self.assertFalse(odd(2))
        self.assertFalse(odd(10))
        self.assertFalse(odd(84))
        self.assertFalse(odd(-48))

        # Template

        self.assertTrue(self.render_template('{{ 2|even }}'))
        self.assertTrue(self.render_template('{{ "3"|odd }}'))
        self.assertEqual(self.render_template('{% if number|even %}Even number{% endif %}', {'number': 2}), 'Even number')

    def test_split(self):

        # Raw

        self.assertEqual(['Hello', 'there', 'sir'], split("Hello, there, sir"))
        self.assertEqual(['This', 'used', 'to', 'be', 'a', 'slug'], split("This-used-to-be-a-slug", '-'))

        # Template

        self.assertEqual(u'I-like-coffee-', self.render_template('{% with "I, like, coffee"|split as list %}{% for word in list %}{{ word }}-{% endfor %}{% endwith %}'))
        self.assertEqual(u'You, smell, funny, ', self.render_template('{% with "You-smell-funny"|split:"-" as list %}{% for word in list %}{{ word }}, {% endfor %}{% endwith %}'))

    def test_possessive(self):

        # Raw

        self.assertEqual("George's", possessive("George"))
        self.assertEqual("Chris'", possessive("Chris"))

        # Template

        self.assertEqual("Tom&#39;s", self.render_template('{{ "Tom"|possessive }}'))
        self.assertEqual("Sally&#39;s", self.render_template('{{ name|possessive }}', {'name': 'Sally'}))

    def test_jsonify(self):

        data = [
            {
                'id': 1,
                'name': 'Bill'
            },
            {
                'id': 2,
                'name': 'Sam'
            },
            {
                'id': 3,
                'name': 'Mr. Giggles'
            }
        ]

        data_string = u'[{"id": 1, "name": "Bill"}, {"id": 2, "name": "Sam"}, {"id": 3, "name": "Mr. Giggles"}]'

        self.assertEqual(data_string, self.render_template("{{ data|jsonify }}", {'data': data}))

    def test_paragraphlist(self):

        text = """Hello, here's the first paragraph.

        Oh, hey. Here's the 2nd paragraph."""

        # Raw

        self.assertEqual(["Hello, here's the first paragraph.", "Oh, hey. Here's the 2nd paragraph."], paragraphlist(text))

        # Template

        text = """
        Hey dude.

        What's up?
        """

        self.assertEqual(u"<p>Hey dude.</p><p>What&#39;s up?</p>", self.render_template('{% for para in text|paragraphlist %}<p>{{ para }}</p>{% endfor %}', {'text': text}))


    def test_multibyte_truncate(self):

        basic_text = '<h1>Oh hey there, fancypants</h1>'
        self.assertEqual("<h1>Oh ...</h1>", multibyte_truncate(basic_text, "1,1"))

        multibyte_text = u"<p>\u3046 \u305f\u3093 \u3041\u3093 \u3042\u3044\u3093 \u306e\u3072\u3093 \u3068 \u3075 \u3046\u3043<p>"
        self.assertEqual(u"<p>\u3046 ...</p>", multibyte_truncate(multibyte_text, "0,1"))


class ColumnizeTagTestCase(TemplateTestCase):
    
    """
    Tests for 'columnize' template tag
    """
    
    global_template_string = '{% load geetar %}'
    
    def test_basic(self):
        
        # Basic
        
        mylist = ['hello', 'there', 'dude']
        expr = '{% columnize mylist into 3 %}'
        rendered, context = self.render(expr, {'mylist': mylist})
        
        self.assertEqual(len(context['mylist']), 3, "The length of 'mylist' in the resulting context is not 3")
        
        self.assertEqual(len(context['mylist'][0]), 1, "The first column's length is not equal to 1")
        self.assertEqual(context['mylist'][0][0], 'hello', "'hello' was not properly sorted into the first column")
        
        self.assertEqual(len(context['mylist'][1]), 1, "The second column's length is not equal to 1")
        self.assertEqual(context['mylist'][1][0], 'there', "'there' was not properly sorted into the second column")
        
        self.assertEqual(len(context['mylist'][2]), 1, "The third column's length is not equal to 1")
        self.assertEqual(context['mylist'][2][0], 'dude', "'dude' was not properly sorted into the third column")
        
        # Unequal column distribution
        
        mylist = ['hello', 'there', 'dude']
        expr = '{% columnize mylist into 2 %}'
        rendered, context = self.render(expr, {'mylist': mylist})
        
        self.assertEqual(len(context['mylist']), 2, "The length of 'mylist' in the resulting context is not 2")
        
        self.assertEqual(len(context['mylist'][0]), 2, "The first column's length is not equal to 2")
        self.assertEqual(context['mylist'][0][0], 'hello', "'hello' was not properly sorted into the first column")
        self.assertEqual(context['mylist'][0][1], 'dude', "'dude' was not properly sorted into the first column")
        
        self.assertEqual(len(context['mylist'][1]), 1, "The second column's length is not equal to 1")
        self.assertEqual(context['mylist'][1][0], 'there', "'there' was not properly sorted into the second column")
    
    def test_as_keyword(self):
        
        mylist = ['some', 'test', 'list', 'dude']
        expr = '{% columnize mylist into 3 as differentlist %}'
        rendered, context = self.render(expr, {'mylist': mylist})
        
        self.assertEqual(context['mylist'], mylist, "The original 'mylist' differs from the context's 'mylist'")
        self.assertEqual(len(context['differentlist']), 3, "The length of 'differentlist' in the resulting context is not 3")
        
        self.assertEqual(len(context['differentlist'][0]), 2, "The first column's length is not equal to 2")
        self.assertEqual(context['differentlist'][0][0], 'some', "'some' was not properly sorted into the first column")
        self.assertEqual(context['differentlist'][0][1], 'dude', "'dude' was not properly sorted into the first column")
        
        self.assertEqual(len(context['differentlist'][1]), 1, "The second column's length is not equal to 1")
        self.assertEqual(context['differentlist'][1][0], 'test', "'test' was not properly sorted into the second column")
        
        self.assertEqual(len(context['differentlist'][2]), 1, "The third column's length is not equal to 1")
        self.assertEqual(context['differentlist'][2][0], 'list', "'test' was not properly sorted into the third column")
    
    def test_stacked_keyword(self):
        
        mylist = ['some', 'test', 'list', 'dude']
        expr = '{% columnize mylist into 3 stacked %}'
        rendered, context = self.render(expr, {'mylist': mylist})
        
        self.assertEqual(len(context['mylist']), 3, "The length of 'mylist' in the resulting context is not 3")
        
        self.assertEqual(context['mylist'][0], ['some', 'test'], "First column was not sorted properly")
        self.assertEqual(context['mylist'][1], ['list'], "Second column was not sorted properly")
        self.assertEqual(context['mylist'][2], ['dude'], "Third column was not sorted properly")
        
        # Longer list
        
        mylist = ['some', 'list', 'to', 'test', 'with', 'and', 'such', 'dude', 'awesome']
        expr = '{% columnize mylist into 4 stacked as testlist %}'
        rendered, context = self.render(expr, {'mylist': mylist})
        
        self.assertEqual(context['mylist'], mylist, "The value of 'mylist' changed in the resulting context")
        self.assertEqual(len(context['testlist']), 4, "The length of 'testlist' is incorrect")
        
        self.assertEqual(len(context['testlist'][0]), 3, "The first column's length is incorrect")
        self.assertEqual(context['testlist'][0], ['some', 'list', 'to'], "The first column was not sorted properly")
        
        self.assertEqual(len(context['testlist'][1]), 2, "The second column's length is incorrect")
        self.assertEqual(context['testlist'][1], ['test', 'with'], "The second column was not sorted properly")
        
        self.assertEqual(len(context['testlist'][2]), 2, "The third column's length is incorrect")
        self.assertEqual(context['testlist'][2], ['and', 'such'], "The third column was not sorted properly")
        
        self.assertEqual(len(context['testlist'][3]), 2, "The fourth column's length is incorrect")
        self.assertEqual(context['testlist'][3], ['dude', 'awesome'], "The fourth column was not sorted properly")


class ValueFromSettingsTestCase(TemplateTestCase):

    global_template_string = '{% load geetar %}'

    def test_basic(self):

        expr = '{% value_from_settings SECRET_KEY %}'
        rendered, context = self.render(expr)

        self.assertEqual(rendered, 'geetarT3st!')



from math import floor
import re

from django import template
from django.conf import settings
from django.template.base import Node
from django.template.defaultfilters import stringfilter
from django.utils import simplejson
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from django.utils.text import Truncator


register = template.Library()


# ---- Filters


@register.filter
def even(value):
    return int(value) % 2 == 0


@register.filter
def odd(value):
    return not even(value)


@register.filter(is_safe=False)
@stringfilter
def split(value, character=','):

    """
    Simply calls built-in .split() method on the string, splitting it by the
    passed-in character

    Example

        list = 'Hey, there, dudes'
        {{ list|split:',' }} yields ['Hey', 'there', 'dudes']
    """

    return [bit.strip() for bit in value.split(character)]


@register.filter(is_safe=False)
@stringfilter
def possessive(value):

    """
    Possessive template filter. Takes a name and appends "'s" or "'" to the
    end depending on whether or not the name ends with an 's'

    Example:

        name = 'Sally'
        {{ name|possessive }} yields "Sally's"

        name = 'Chris'
        {{ name|possessive }} yields "Chris'"

    """

    return "%s'%s" % (value, '' if value.rstrip()[-1] == 's' else 's')


@register.filter
def jsonify(data):

    """
    Simple template tag proxy to simplejson.dumps()
    """

    return mark_safe(simplejson.dumps(data))


# Hella robbed from:
# http://www.holovaty.com/writing/django-two-phased-rendering/
# Adds {% raw %} / {% endraw %} template tag so you can throw in
# Django-template-like content that won't be parsed (e.g. Mustache)
#
# Copyright 2009, EveryBlock
# This code is released under the GPL.

@register.tag
def raw(parser, token):
    # Whatever is between {% raw %} and {% endraw %} will be preserved as
    # raw, unrendered template code.
    text = []
    parse_until = 'endraw'
    tag_mapping = {
        template.TOKEN_TEXT: ('', ''),
        template.TOKEN_VAR: ('{{', '}}'),
        template.TOKEN_BLOCK: ('{%', '%}'),
        template.TOKEN_COMMENT: ('{#', '#}'),
    }
    # By the time this template tag is called, the template system has already
    # lexed the template into tokens. Here, we loop over the tokens until
    # {% endraw %} and parse them to TextNodes. We have to add the start and
    # end bits (e.g. "{{" for variables) because those have already been
    # stripped off in a previous part of the template-parsing process.
    while parser.tokens:
        token = parser.next_token()
        if (token.token_type == template.TOKEN_BLOCK and token.contents == parse_until):
            return template.TextNode(u''.join(text))
        start, end = tag_mapping[token.token_type]
        text.append(u'%s%s%s' % (start, token.contents, end))
    parser.unclosed_block_tag(parse_until)


@register.filter(is_safe=False)
@stringfilter
def paragraphlist(value):

    """
    Takes a string and breaks it apart by newline/return characters and
    returns a list of paragraphs

    Example:

        'Hello there,

        My name is $horty'

        would yield:

        ['Hello there,', 'My name is $horty']

    """

    # Normalize newlines

    value = re.sub(r'[ \t]*(\r\n|\r|\n)[ \t]*', '\n', force_unicode(value.strip()))
    return re.split('\n{2,}', value)


@register.filter(is_safe=True)
def multibyte_truncate(value, arg):
    """
    Truncates (potentially multibyte) chunk of HTML after a certain number of words.

    Argument: A comma-separated string containing two numbers. The first is the
    number of words to truncate for utf-8 strings, the second for multibyte
    characters.

    Newlines in the HTML are preserved.
    """

    # workaround for the fact that filters can only take one arg
    args = [a.strip() for a in arg.split(',')]

    try:
        utf_length = int(args[0])
        multibyte_length = int(args[1])
    except ValueError: # invalid literal for int()
        return value # Fail silently.

    try:
        # this will fail for multibyte strings
        value.decode('utf-8')
    except UnicodeEncodeError:
        return Truncator(value).words(multibyte_length, html=True, truncate=' ...')

    return Truncator(value).words(utf_length, html=True, truncate=' ...')


# ---- Template Tags


class ColumnizeNode(Node):

    """
    Handle 'columnize' tag parsing
    """

    def __init__(self, expression, target, columns, stacked=False):

        """
        Columnize init

        `expression`    The FilterExpression instance for the source to
                        columnize from
        `target`        The target context variable for the resulting,
                        sorted/columnized data
        `columns`       The number of columns to sort the source data into
        `stacked`       Whether the data should be sorted in a stacked manner
                        or not. By default the data is sorted into columns by
                        alternating through the source, setting this to True
                        would break the contents of the source into columns,
                        maintaining its initial order.

        """

        self.expression = expression
        self.target = target
        self.columns = columns
        self.stacked = stacked

    def render(self, context):

        source_list = self.expression.resolve(context, True)

        if not source_list:
            context[self.target] = []
            return ''

        out = [[] for i in range(self.columns)]
        lengths = [0 for i in range(self.columns)]

        # Figure out how long each column should be

        if self.stacked:
            total = len(source_list)
            rem = total % self.columns

            if total > self.columns:
                lengths = [int(floor(float(total) / float(self.columns)))] * self.columns

            if rem:
                i = 0
                while rem:
                    lengths[i] += 1
                    rem -= 1
                    i += 1

        # Sort into columns

        i = 0
        column = 0

        for item in source_list:

            out[column].append(item)
            i += 1

            if self.stacked:
                if len(out[column]) == lengths[column]:
                    column += 1
            else:
                column = i % self.columns

        context[self.target] = out

        return ''


@register.tag(name='columnize')
def do_columnize(parser, token):

    """
    Take an iterable and sort the contents into the specified number of
    columns/buckets. By default, the contents are sorted by alternating
    columns, but the 'stacked' keyword can be used to break the contents into
    columns in the order that they appear in the iterable. So for example, the
    default behavior would look like this:

        mylist = [1, 2, 3, 4, 5]

        {% columnize mylist into 3 %}
        ...
        mylist = [[1, 4], [2, 5], [3]]

    Using the 'stacked' keyword looks like this:

        mylist = [1, 2, 3, 4, 5]

        {% columnize mylist into 3 stacked %}
        ...
        mylist = [[1, 2], [3, 4], [5]]

    Usage:

        Separate the contents of 'some_var' into 3 columns:

        {% columnize some_var into 3 %}

        Separate the contents of 'my_var' into 4 columns and store as
        'columnated':

        {% columnize my_var into 4 as columnated %}

        Separate the contents of 'rad_var' into 5 columns in its original
        order (stacked):

        {% columnize rad_var into 5 stacked %}

        Separate the contents of 'yay_var' into 8 columns in its original
        order (stacked) and store as 'yay_columns':

        {% columnize yay_var into 8 stacked as yay_columns %}

    """

    bits = token.contents.split()
    bits.pop(0) # Pop off 'columnize' bit

    if len(bits) < 3:
        raise template.TemplateSyntaxError("'columnize' template tag takes at least 3 arguments")

    source = bits.pop(0) # Store first argument as the 'source' variable

    # Validate 'into #' portion

    if bits[0] != 'into':
        raise template.TemplateSyntaxError("The 2nd argument for 'columnize' must be 'into', followed by the number of columns you want to divide into")

    bits.pop(0)

    try:
        columns = int(bits.pop(0))
    except:
        raise template.TemplateSyntaxError("The 3rd argument for 'columnize' must be numeric, indicating how many columns you want your variable divied into")

    # Validate optional 'stacked' keyword

    stacked = False

    if bits and bits[0] == 'stacked':
        stacked = True
        bits.pop(0)

    if bits:
        if len(bits) == 2 and bits[-2] == 'as':
            target = bits[-1]
        else:
            raise template.TemplateSyntaxError("The last 2 arguments of 'columnize' must be empty, or 'as' followed by the variable name you want the resulting columnized data stored into")
    else:
        target = source

    # Compile source expresion

    expression = parser.compile_filter(source)

    return ColumnizeNode(expression, target, columns, stacked)


@register.tag
def value_from_settings(parser, token):

    """
    Get a value from the global settings. Useful for retrieving
    environment-specific settings directly from the template.

    Usage:
        {% value_from_settings BLOG_URL %}
    """

    try:
        tag_name, var = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires a single argument" % token.contents.split()[0]
    return ValueFromSettings(var)


class ValueFromSettings(template.Node):

    def __init__(self, var):
        self.arg = template.Variable(var)

    def render(self, context):
        return settings.__getattr__(str(self.arg))



from django.template import Context, Template
from django.utils import unittest


class TemplateTestCase(unittest.TestCase):
    
    """
    Base test case for template stuff with some handy render shortcuts
    """
    
    global_template_string = '' # Optional template string to include with all renders, good for calling {% load ... %} and such
    
    def render(self, template, context={}):

        """
        Returns render result and context from passed in template string and context
        
            `template`  string/list/tuple   String or strings to be rendered as a template
            `context`   dict                Context variables to render into template
        
        """
        
        strings = [self.global_template_string]
        
        if type(template) in (list, tuple):
            strings.extend(list(template))
        else:
            strings.append(template)
        
        template = ''.join(strings)
        
        rendered = Template(template).render(Context(context))
        
        return rendered, context

    def render_template(self, template, context={}):
        
        """
        Returns just the render result of a template string and context combination
        
            `template`  string/list/tuple   String or strings to be rendered as a template
            `context`   dict                Context variables to render into template
        
        """
        
        rendered, context = self.render(template, context)
        return rendered


import os

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = (
    'discover_runner',
    'geetar',
    'test_app',
)

SECRET_KEY = "geetarT3st!"

TEST_RUNNER = 'discover_runner.DiscoverRunner'

TEST_DISCOVER_ROOT = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'tests')

GOOGLE_MAPS_API_KEY = 'AIzaSyCuJe82BXhTESkKmlo3V4Rft_IsxGhJwDU'


from django.contrib import admin


class StatusableAdminMixin(object):

    """
    Simple admin for StatusableModel instances adding batch
    activate/deactivate functionality
    """

    actions = ['activate', 'deactivate']

    def _result_string(self, result):
        plural = not result == 1
        return '%d Item%s %s' % (result, 's' if plural else '',
                                   'were' if plural else 'was',)

    def activate(self, request, queryset):
        result = queryset.activate()
        msg = self._result_string(result)
        self.message_user(request, '%s activated' % msg)

    activate.short_description = 'Activate selected %(verbose_name_plural)s'

    def deactivate(self, request, queryset):
        result = queryset.deactivate()
        msg = self._result_string(result)
        self.message_user(request, '%s deactivated' % msg)

    deactivate.short_description = 'Deactivate selected %(verbose_name_plural)s'



from django.utils import unittest

from geetar.paginator import WindowedPaginator, WindowedPage


class WindowedPaginatorTestCase(unittest.TestCase):
    
    def test_windowed_paginator(self):

        items = range(0, 86)
        pager = WindowedPaginator(items, 5) # 86 objects, 18 pages

        # ---- Odd window

        window = 5
        page = pager.page(1, 5)

        self.assertIsInstance(page, WindowedPage)

        # Test sets by window size, then a tuple: index 0 is pages that should equate to the page range in index 1

        test_sets = {
            4: (
                ([1,2], [1, 2, 3, 4, 5, None, 18],), # Lower end
                ([6], [1, None, 5, 6, 7, 8, None, 18],), # Middle
                ([16, 17, 18], [1, None, 14, 15, 16, 17, 18],), # Higher
            ),
            5: (
                ([1, 2, 3], [1, 2, 3, 4, 5, 6, None, 18],), # Lower end
                ([5], [1, None, 3, 4, 5, 6, 7, None, 18],), # Middle
                ([16, 17, 18], [1, None, 13, 14, 15, 16, 17, 18]) # Higher
            ),
            18: (
                (range(1, 19), range(1, 19),),
            )
        }

        for window in test_sets.keys():
            for test in test_sets[window]:
                for num in test[0]:
                    page = pager.page(num, window=window)
                    self.assertIsInstance(page, WindowedPage, 'Instance is not a `WindowedPage` object')
                    self.assertEqual(page.page_range, test[1], 'Window %d, page %d page range incorrect' % (window, num))


import math

from django.core.paginator import Paginator, Page


"""
Pagination classes for windowed/ranged pagination.
These classes build on the pagination built in to Django and add functionality
so you can paginate a la:

    1 ... 5, 6, 7, 8 ... 32

See https://docs.djangoproject.com/en/1.4/topics/pagination/ for general
usage--we simply add a `window` argument to the class constructors for
Paginator/Page on top of what they already support.

"""


class WindowedPaginator(Paginator):

    def page(self, number, window=None):

        """
        Returns a Page object for the given 1-based page number.
        This is a copy of Paginator.page, but returning a WindowedPage
        instance instead of a Page one
        """

        number = self.validate_number(number)
        bottom = (number - 1) * self.per_page
        top = bottom + self.per_page
        if top + self.orphans >= self.count:
            top = self.count
        return WindowedPage(self.object_list[bottom:top], number, self,
                            window=window)


class WindowedPage(Page):

    def __init__(self, object_list, number, paginator, window=None):

        """
        WindowedPage object constructor

        `window`    integer     A number indicating how many page links
                                you want to display in between the first and
                                last pages. Example: a value of '5' with a
                                total of 30 pages, on page 14 would yield a
                                page range of:
                                [1, None, 12, 13, 14, 15, 16, None, 30]
        """

        super(WindowedPage, self).__init__(object_list, number, paginator)
        self.window = window

    def _get_page_range(self):

        # If a window is set, and that window plus the first and last pages
        # is less than the total...

        if self.window and self.window + 2 < self.paginator.num_pages:

            # Calculate lower and higher window limits

            page_max = self.paginator.num_pages
            offset = int(math.floor(float(self.window - 1) / 2))
            lower = self.number - offset
            higher = self.number + (self.window - offset)

            if lower <= 2:
                lower = 2
                higher = 2 + self.window
            elif higher >= page_max:
                higher = page_max
                lower = higher - self.window

            # Build pages list

            pages = [1]

            if lower > 2:
                pages.append(None)

            pages.extend(range(lower, higher))

            if higher < page_max - 1:
                pages.append(None)

            pages.append(page_max)

            return pages
        else:
            return self.paginator._get_page_range()

    page_range = property(_get_page_range)



from setuptools import setup


setup(
    name="django-geetar",
    version=__import__('geetar').get_version().replace(' ', '-'),
    packages=[
        'geetar',
        'geetar.statusable',
        'geetar.templatetags',
        'geetar.tests'
    ],
    author="Instrument",
    author_email="chris.forrette@weareinstrument.com",
    description="A swiss army knife for your Django project, including a smattering of helpful code",
    keywords="django, helpers",
    url="https://github.com/Instrument/django-geetar",
    include_package_data=True,
    install_requires=[
        'django>=1.4'
    ],
    classifiers=[
        'Framework :: Django',
        'Topic :: Internet',
        'License :: OSI Approved :: BSD License',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7'
    ]
)




import os
from django.utils import unittest
from django.core.files import File
from django.forms import forms

from geetar.fields import RestrictedTypeFileField


class RestrictedTypeFileFieldTestCase(unittest.TestCase):


    def test_no_accepted_types_field(self):

        field = RestrictedTypeFileField()

        f = File(open(os.path.join(os.path.abspath(os.path.dirname(__file__)), '__init__.py')))
        f.content_type = 'image/bmp'
        data = type("",(object,),dict(file=f))

        with self.assertRaises(forms.ValidationError):
            field.clean(data, None)


    def test_unaccepted_types_field(self):

        accepted_types = ['image/bmp',]
        field = RestrictedTypeFileField(accepted_types=accepted_types)

        f = File(open(os.path.join(os.path.abspath(os.path.dirname(__file__)), '__init__.py')))
        f.content_type = 'application/exe'
        data = type("",(object,),dict(file=f, accepted_types=accepted_types))

        with self.assertRaises(forms.ValidationError):
            field.clean(data, None)


    def test_accepted_types_field(self):

        accepted_types = ['image/bmp','image/jpg', 'image/png']
        field = RestrictedTypeFileField(accepted_types=accepted_types)

        f = File(open(os.path.join(os.path.abspath(os.path.dirname(__file__)), '__init__.py')))
        f.content_type = 'image/bmp'
        data = type("",(object,),dict(file=f, accepted_types=accepted_types))

        self.assertEquals(field.clean(data, {}).file, f)




from urllib import urlencode
from urllib2 import urlopen

from django.conf import settings
from django.utils import simplejson


"""
Handy Google maps geocode search functions
"""


def find_geo(location, api_key=None):

    """
    Query Google maps for geo data on the passed in `location`.
    Returns dictionary of data if successful, otherwise None

    `location`  string  An address, whether partial or complete, to query
                        Google Maps about
    `api_key`   string  Google Maps API key. If this is left empty,
                        GOOGLE_MAPS_API_KEY setting is used instead
    """

    data = {
        'q': location,
        'output': "json",
        'oe': "utf8",
        'sensor': "false",
        'key': api_key or settings.GOOGLE_MAPS_API_KEY
    }

    url = "http://maps.google.com/maps/geo?%s" % urlencode(data)
    response = urlopen(url)
    geo_content = simplejson.loads(response.read())

    return geo_content if geo_content['Status']['code'] == 200 else None


def find_geo_point(location, api_key=None):

    """
    Use `find_geo` to query Google Maps and returns a tuple containing the
    normalized address and a point tuple containing the latitude and longitude
    points, or returns False if no data is found

    `location`  string  An address, whether partial or complete, to query
                        Google Maps about
    `api_key`   string  Google Maps API key. If this is left empty,
                        GOOGLE_MAPS_API_KEY setting is used instead
    """

    geo_content = find_geo(location, api_key)

    if geo_content:
        placemark = geo_content['Placemark'][0]
        lng, lat = placemark['Point']['coordinates'][:2]
        return (placemark['address'], (lng, lat,),)
    else:
        return geo_content



from django.db import models

from geetar.statusable.models import StatusableModel, SingleActiveModel


class Author(StatusableModel):

    first_name = models.CharField(max_length=255)
    last_name = models.CharField(max_length=255)


class Book(StatusableModel):

    author = models.ForeignKey(Author)
    name = models.CharField(max_length=255)


class FeaturedAuthor(SingleActiveModel):

    author = models.ForeignKey(Author)


from django.db import models


"""
Base model and accompanying QuerySet and Manager for 'statusable' models.
Adds a 'status' field to the model along with
"""


STATUS_INACTIVE = 1
STATUS_ACTIVE = 2
STATUS_CHOICES = (
    (STATUS_INACTIVE, 'Inactive',),
    (STATUS_ACTIVE, 'Active',),
)


class StatusableQuerySet(models.query.QuerySet):

    """
    QuerySet class for statusable objects with some handy shortcut queries
    """

    def inactive(self):
        return self.filter(status=STATUS_INACTIVE)

    def deactivate(self):
        return self.update(status=STATUS_INACTIVE)

    def active(self):
        return self.filter(status=STATUS_ACTIVE)

    def activate(self):
        return self.update(status=STATUS_ACTIVE)


class StatusableManager(models.Manager):

    """
    Object manager to be paired with StatusableModel to add a few handy query
    shortcuts. This manager needs to work as a proxy for any custom QuerySet
    methods so that chaining is still possible.
    """

    def get_query_set(self):
        return StatusableQuerySet(self.model, using=self._db)

    def inactive(self):
        return self.get_query_set().inactive()

    def deactivate(self):
        return self.get_query_set().deactivate()

    def active(self):
        return self.get_query_set().active()

    def activate(self):
        return self.get_query_set().activate()


class StatusableModel(models.Model):

    """
    Base model for models that require status-able state
    """

    objects = StatusableManager()

    status = models.IntegerField(db_index=True, choices=STATUS_CHOICES,
                                 default=STATUS_INACTIVE)

    class Meta:
        abstract = True

    def deactivate(self, save=True):
        self.status = STATUS_INACTIVE
        if save:
            self.save()

    def activate(self, save=True):
        self.status = STATUS_ACTIVE
        if save:
            self.save()

    @property
    def is_inactive(self):
        return not self.is_active

    @property
    def is_active(self):
        return self.status == STATUS_ACTIVE


class SingleActiveQuerySet(StatusableQuerySet):

    """
    Make sure that when records are bulk activated, only 1 is actually
    activated
    """

    def active(self):
        return self.get(status=STATUS_ACTIVE)

    def activate(self):

        """
        If this QuerySet contains multiple records, just pull off the first
        and activate it
        """

        rows = self.all()
        if len(rows):
            active = rows[0]
            return active.activate()
        return 0


class SingleActiveManager(StatusableManager):

    """
    Simply setting the SingleActiveQuerySet
    """

    def get_query_set(self):
        return SingleActiveQuerySet(self.model, using=self._db)


class SingleActiveModel(StatusableModel):

    """
    Extension of the statusable model that only allows one record to be active
    at a time
    """

    objects = SingleActiveManager()

    # Optional list of fields to make active status unique with

    _active_with_fields = []

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):

        """
        After save, if this record is active, deactivate all others
        """

        super(SingleActiveModel, self).save(*args, **kwargs)

        if self.status == STATUS_ACTIVE:
            active = self.__class__.objects.filter(status=STATUS_ACTIVE).exclude(pk=self.pk)
            if self._active_with_fields:
                filters = {}
                for field in self._active_with_fields:
                    filters[field] = getattr(self, field)
                active = active.filter(**filters)
            active.deactivate()
